unit Unit1;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Forms, Controls, Graphics, Dialogs, LCLIntf, StdCtrls,
  ExtCtrls, LCLProc, Spin;


type

  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    SpinEdit1: TSpinEdit;
    Timer1: TTimer;
    procedure Button1Click({%H-}Sender: TObject);
    procedure FormCreate({%H-}Sender: TObject);
    procedure FormDestroy({%H-}Sender: TObject);
    procedure FormPaint({%H-}Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure Timer1Timer({%H-}Sender: TObject);
  private
    l          : integer;
    angle      : integer;
    aDirection : integer;
    aPicture   : array [0..4] of TPicture;
    f          : double;
    d          : double;
    bground    : integer;
  end;

var
  Form1: TForm1;

implementation
{$R *.lfm}
uses
  Math, FPImage, IntfGraphics;

function RotatePoint(const APoint: TPoint; AAngle: Double): TPoint;
var
  sa, ca: Double;
begin
  sa := sin(AAngle);
  ca := cos(AAngle);
  Result.X := Round( ca * APoint.X + sa * APoint.Y);
  Result.Y := Round(-sa * APoint.X + ca * APoint.Y);
end;

function RotateImage(ASourceImg: TLazIntfImage; Angle: Double;Direction: integer): TLazIntfImage;
var
  x, y: Integer;
  C, P: TPoint;
begin
  Result := TLazIntfImage.CreateCompatible(ASourceImg,ASourceImg.Width, ASourceImg.Height);
  C := Point(ASourceImg.Width div 2, ASourceImg.Height div 2);
  for y := 0 to Result.Height-1 do
    for x := 0 to Result.Width-1 do
    begin
      P := RotatePoint(Point(x, y) - C, Angle*Direction) + C;
      P := P - Point((Result.Width - ASourceImg.Width) div 2, (Result.Height - ASourceImg.Height) div 2);
     if (P.X >= 0) and (P.Y >= 0) and (P.X < ASourceImg.Width) and (P.Y < ASourceImg.Height) then
      Result.Colors[x, y] := ASourceImg.Colors[P.X, P.Y]
     else
        Result.Colors[x, y] := colTransparent;
    end;
end;

procedure Blend_Bmp(var BackBmp: TBitmap; const StartBmp,EndBmp: TBitmap; aFrac: double);
var
  x, y     : Integer;
  r,g, b   : Word;
  BackImg  : TLazIntfImage;
  StartImg : TLazIntfImage;
  EndImg   : TLazIntfImage;
begin

 BackImg  := BackBmp.CreateIntfImage;
 StartImg := StartBmp.CreateIntfImage;
 EndImg   := EndBmp.CreateIntfImage;
 try
    for y := 0 to StartBmp.Height-1 do
      for x := 0 to StartBmp.Width - 1 do
      begin
        if StartImg.Colors[x, y] <> colTransparent then
         begin
        r := round((1.0-aFrac) * StartImg.Colors[x, y].Red   + aFrac * EndImg.Colors[x, y].Red);
        g := round((1.0-aFrac) * StartImg.Colors[x, y].Green + aFrac * EndImg.Colors[x, y].Green);
        b := round((1.0-aFrac) * StartImg.Colors[x, y].Blue  + aFrac * EndImg.Colors[x, y].Blue);
        BackImg.Colors[x, y] := FPColor(r, g, b);
         end;
      end;

    BackBmp.LoadFromIntfImage(BackImg);

  finally
    BackImg.Free;
    StartImg.Free;
    EndImg.Free;
  end;
end;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var lv : integer;
begin
 l          := 20;
 d          := 20;
 angle      :=  0;
 aDirection := -1;
 f          :=  0;
 bground    :=  0;
 for lv := 0 to 4 do
  aPicture[lv]  := TPicture.Create;
 aPicture[0].LoadFromResourceName(HInstance,'BGROUNDRED');
 aPicture[1].LoadFromResourceName(HInstance,'CIRCLE_WHITE');
 aPicture[2].LoadFromResourceName(HInstance,'EXIT_03_16');
 aPicture[3].LoadFromResourceName(HInstance,'OK_02_16');
 aPicture[4].LoadFromResourceName(HInstance,'BGROUNDGREEN');
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
 if aDirection = -1 then
  begin
   aDirection := 1;

  end
 else
  begin
   aDirection := -1;
  end;
 Timer1.Enabled := true;
end;

procedure TForm1.FormDestroy(Sender: TObject);
var lv : integer;
begin
 for lv := 0 to 4 do
  aPicture[lv].Free;
end;

procedure TForm1.FormPaint(Sender: TObject);
var
  imgSrc, imgDest   : TLazIntfImage;
  imgSrc1, imgDest1 : TLazIntfImage;
  bmp,bmp1          : TBitmap;
begin
 Canvas.Draw(20,20,aPicture[bground].Graphic);

 aPicture[1].Bitmap.TransparentColor:= clFuchsia;
 aPicture[1].Bitmap.Transparent:= true;
 Canvas.Draw(l,20,aPicture[1].Bitmap);

 imgSrc := aPicture[2].PNG.CreateIntfImage;
 imgDest := RotateImage(imgSrc, DegToRad(angle),aDirection);
 bmp := TBitmap.Create;
 bmp.Pixelformat := pf32Bit;
 bmp.LoadFromIntfImage(imgDest);

 imgSrc1 := aPicture[3].PNG.CreateIntfImage;
 imgDest1 := RotateImage(imgSrc1, DegToRad(angle),aDirection);
 bmp1 := TBitmap.Create;
 bmp1.Pixelformat := pf32Bit;
 bmp1.LoadFromIntfImage(imgDest1);


 Blend_Bmp(bmp,bmp,bmp1,f);

 Canvas.Draw(l+5,25,bmp);
 bmp.Free;
 bmp1.Free;
 imgDest.Free;
 imgSrc.Free;
 imgDest1.Free;
 imgSrc1.Free;

end;

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
 Timer1.Interval:= SpinEdit1.Value;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var i : integer;
begin
 i := 21;
 inc(angle,i);

 if aDirection = 1 then
  begin
   d := d+(35/(360/i));
   l := round(d);
   f := f + (1/(360/i));
   if f > 1 then f := 1;
   if angle > 180 then bground := 4;
   if angle >= 360 then
   begin
    angle := 0;
    l:= 55;
    Timer1.Enabled:= false;
   end;
  end;
 if aDirection = -1 then
  begin
   d := d-(35/(360/i));
   l := round(d);
   f := f - (1/(360/i));
   if f < 0 then f := 0;
   if angle > 180 then bground := 0;
   if angle >= 360 then
   begin
    angle := 0;
    l:= 20;
    Timer1.Enabled:= false;
   end;
  end;
 Invalidate;
end;

end.

